// ignore_for_file: public_member_api_docs

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

part 'channel_with_pivot.g.dart';

/// [ChannelWithPivot] is a widget drawing an ecg sequence per channel.
@CopyWith()
class ChannelWithPivot extends StatelessWidget {
  /// [ChannelWithPivot] is a widget that draw a graph of
  /// an ecg sequence by a given [ecgSequence].
  const ChannelWithPivot({
    super.key,
    required this.ecgSequence,
    this.samplingRate = 250,
    this.strokeWidth = 1.5,
    this.color = Colors.black,
    this.measureTime = 60,
    this.maxY = 2,
    this.minY = -2,
    this.pivot = 0,
    this.pivotColor = Colors.transparent,
    this.pivotStrokeWidth = 0,
    this.backgroundColor = Colors.transparent,
    this.isTappable = false,
  });

  final double measureTime;
  final double strokeWidth;
  final int samplingRate;
  final List<FlSpot> ecgSequence;

  final double pivot;
  final Color pivotColor;
  final double pivotStrokeWidth;

  final Color backgroundColor;

  final double maxY;
  final double minY;

  final Color color;

  final bool isTappable;

  @override
  Widget build(BuildContext context) {
    if (ecgSequence.isEmpty) return const SizedBox();
    final tic = samplingRate * measureTime;

    const minX = 0.0;
    final maxX = tic;

    return LineChart(
      LineChartData(
        minX: minX,
        maxX: maxX,
        minY: minY,
        maxY: maxY,
        // minY: isFiltered ? minY : null,
        // maxY: isFiltered ? maxY : null,
        backgroundColor: backgroundColor,
        lineTouchData: LineTouchData(
          enabled: isTappable,
          // getTouchedSpotIndicator: (barData, spotIndexes) {
          //   return spotIndexes
          //       .map((e) => TouchedSpotIndicatorData(
          //             FlLine(strokeWidth: 0),
          //             FlDotData(),
          //           ))
          //       .toList();
          // },
          touchTooltipData: LineTouchTooltipData(
            tooltipBgColor: Colors.greenAccent.withOpacity(0.1),
            tooltipBorder: BorderSide(color: Colors.black),
            // getTooltipItems: (touchedSpots) {
            //   return touchedSpots
            //       .map(
            //         (e) => LineTooltipItem(
            //           '${e.props.last}',
            //           // '${e.y.toStringAsFixed(4)} mV',
            //           TextStyle(),
            //         ),
            //       )
            //       .toList();
            // },
          ),
        ),
        gridData: FlGridData(
          show: false,
        ),
        titlesData: FlTitlesData(
          show: false,
        ),
        borderData: FlBorderData(
          show: false,
        ),
        // clipData: FlClipData.all(),
        lineBarsData: [
          LineChartBarData(
            spots: ecgSequence,
            color: color,
            isCurved: false,
            barWidth: strokeWidth,
            dotData: FlDotData(show: false),
          ),
          if (pivotStrokeWidth != 0 && pivot != 0)
            LineChartBarData(
              spots: [
                FlSpot(pivot, minY),
                FlSpot(pivot, maxY),
              ],
              color: pivotColor,
              isCurved: false,
              barWidth: pivotStrokeWidth,
              dotData: FlDotData(show: false),
            ),
        ],
      ),
      chartRendererKey: UniqueKey(),
      // swapAnimationDuration: Duration.zero,
    );
  }
}
